from collections import deque, defaultdict

def main():
    import sys
    input = sys.stdin.read
    data = input().splitlines()

    # Считываем основные параметры
    n, m, k = map(int, data[0].split())

    # Создаем граф в виде списка смежности
    graph = defaultdict(list)
    for i in range(1, m + 1):
        a, b = map(int, data[i].split())
        graph[a].append(b)
        graph[b].append(a)

    # Считываем охранников
    guards = []
    for i in range(m + 1, m + k + 1):
        p, h = map(int, data[i].split())
        guards.append((p, h))

    # Массив для хранения минимального расстояния от охранников до складов
    INF = float('inf')
    distances = [INF] * (n + 1)

    # Обрабатываем каждого охранника через BFS
    for start, endurance in guards:
        if distances[start] > endurance:
            # BFS с ограничением по расстоянию
            queue = deque([(start, 0)])  # (текущий склад, расстояние от охранника)
            while queue:
                node, dist = queue.popleft()

                if dist > endurance or dist >= distances[node]:
                    continue  # Прекращаем обработку, если выносливость превышена

                distances[node] = dist  # Обновляем минимальное расстояние

                # Добавляем соседей в очередь
                for neighbor in graph[node]:
                    if distances[neighbor] > dist + 1:
                        queue.append((neighbor, dist + 1))

    # Список охраняемых складов
    protected = [i for i in range(1, n + 1) if distances[i] != INF]
    print(len(protected))
    if protected:
        print(' '.join(map(str, sorted(protected))))
    else:
        print()

if __name__ == "__main__":
    main()
